@@ -1,6 +1,6 @@
 #!/bin/bash
 
-exec &>/tmp/website-$(date +%s).log
+[[ -t 0 ]] || exec &>/tmp/website-$(date +%s).log
 
 TARGET=${1:-/srv/www/vhosts/www.exim.org}
 LATEST=`(cd docbook; ls -1dv [0-9].[0-9]* | tail -1)`
